package cz.itnetwork

import kotlin.math.round

/*
 *  _____ _______         _                      _
 * |_   _|__   __|       | |                    | |
 *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
 *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
 *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
 * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
 *                   ___
 *                  |  _|___ ___ ___
 *                  |  _|  _| -_| -_|  LICENCE
 *                  |_| |_| |___|___|
 *
 * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
 *
 * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
 *
 * Můžete ho upravovat a používat jak chcete, musíte však zmínit
 * odkaz na http://www.itnetwork.cz
 */

/**
 * @param jmeno Jméno bojovníka
 * @param zivot Život v HP
 * @param utok Útok v HP
 * @param obrana Obrana v HP
 * @param kostka Instance hrací kostky
 */
class Bojovnik(private val jmeno: String, private var zivot: Int, private val utok: Int,
               private val obrana: Int, private val kostka: Kostka) {

    /** Maximální zdraví */
    private val maxZivot = zivot

    /** Poslední zpráva */
    private var zprava = ""

    private fun nastavZpravu(zprava: String) {
        this.zprava = zprava
    }

    fun vratPosledniZpravu(): String {
        return zprava
    }

    /**
     * Zjistí, zda je bojovník naživu
     * @return True, pokud je naživu, jinak false
     */
    fun nazivu(): Boolean {
        return (zivot > 0)
    }

    /**
     * Vrátí grafickou reprezentaci života
     * @return Řetězec s grafickou reprezentací života
     */
    fun grafickyZivot(): String {
        var s = "["
        val celkem = 20
        var pocet = round((zivot.toDouble()/maxZivot) * celkem).toInt()
        if ((pocet == 0) && (nazivu()))
            pocet = 1
        s = s.padEnd(pocet + s.length, '#')
        s = s.padEnd(celkem - pocet + s.length, ' ')
        s += "]"
        return s
    }

    /**
     *  Brání se proti úderu soupeře
     * @param uder Úder soupeře v HP
     */
    fun branSe(uder: Int) {
        val zraneni = uder - (obrana + kostka.hod())
        if(zraneni > 0) {
            zivot -= zraneni
            zprava = "$jmeno utrpěl poškození $zraneni hp"
            if (zivot <= 0) {
                zivot = 0
                zprava += " a zemřel"
            }
        } else
            nastavZpravu("$jmeno odrazil útok")
        nastavZpravu(zprava)
    }

    /**
     *  Provede útok na soupeře
     * @param souper Soupeř bojovník
     */
    fun utoc(souper: Bojovnik) {
        val uder = utok + kostka.hod()
        nastavZpravu("$jmeno útočí s úderem za $uder hp")
        souper.branSe(uder)
    }

    /**
     * Vrací textovou reprezentaci bojovníka
     * @return Textová reprezentace bojovníka
     */
    override fun toString(): String {
        return jmeno
    }
}
